Package: robustbase
Version: 0.99-4-1
VersionNote: Released 0.99-4 on 2024-08-19, 0.99-3 on 2024-07-01 and
        0.99-2 on 2024-01-27 to CRAN
Date: 2024-09-24
Title: Basic Robust Statistics
Authors@R: c(person("Martin","Maechler", role=c("aut","cre"), email="maechler@stat.math.ethz.ch", comment = c(ORCID = "0000-0002-8685-9910"))
         , person("Peter", "Rousseeuw", role="ctb", comment = "Qn and Sn")
         , person("Christophe", "Croux", role="ctb", comment = "Qn and Sn")
         , person("Valentin", "Todorov", role = "aut", email = "valentin.todorov@chello.at",
                  comment = "most robust Cov")
         , person("Andreas", "Ruckstuhl", role = "aut", email = "andreas.ruckstuhl@zhaw.ch",
                  comment = "nlrob, anova, glmrob")
         , person("Matias", "Salibian-Barrera", role = "aut", email = "matias@stat.ubc.ca",
                  comment = "lmrob orig.")
         , person("Tobias", "Verbeke", role = c("ctb","fnd"), email = "tobias.verbeke@openanalytics.eu",
                  comment = "mc, adjbox")
         , person("Manuel", "Koller", role = "aut", email = "koller.manuel@gmail.com",
                  comment = "mc, lmrob, psi-func.")
         , person(c("Eduardo", "L. T."), "Conceicao", role = "aut", email = "mail@eduardoconceicao.org",
                  comment = "MM-, tau-, CM-, and MTL- nlrob")
         , person("Maria", "Anna di Palma", role = "ctb", comment = "initial version of Comedian")
          )
URL: https://robustbase.R-forge.R-project.org/,
        https://R-forge.R-project.org/R/?group_id=59,
        https://R-forge.R-project.org/scm/viewvc.php/pkg/robustbase/?root=robustbase,
        svn://svn.r-forge.r-project.org/svnroot/robustbase/pkg/robustbase
BugReports: https://R-forge.R-project.org/tracker/?atid=302&group_id=59
Description: "Essential" Robust Statistics.
 Tools allowing to analyze data with robust methods.  This includes
 regression methodology including model selections and multivariate
 statistics where we strive to cover the book "Robust Statistics,
 Theory and Methods" by 'Maronna, Martin and Yohai'; Wiley 2006.
Depends: R (>= 3.5.0)
Imports: stats, graphics, utils, methods, DEoptimR
Suggests: grid, MASS, lattice, boot, cluster, Matrix, robust,
        fit.models, MPV, xtable, ggplot2, GGally, RColorBrewer,
        reshape2, sfsmisc, catdata, doParallel, foreach, skewt
SuggestsNote: mostly only because of vignette graphics and simulation
Enhances: robustX, rrcov, matrixStats, quantreg, Hmisc
EnhancesNote: linked to in man/*.Rd
LazyData: yes
NeedsCompilation: yes
License: GPL (>= 2)
Packaged: 2024-09-25 09:50:14 UTC; maechler
Author: Martin Maechler [aut, cre] (<https://orcid.org/0000-0002-8685-9910>),
  Peter Rousseeuw [ctb] (Qn and Sn),
  Christophe Croux [ctb] (Qn and Sn),
  Valentin Todorov [aut] (most robust Cov),
  Andreas Ruckstuhl [aut] (nlrob, anova, glmrob),
  Matias Salibian-Barrera [aut] (lmrob orig.),
  Tobias Verbeke [ctb, fnd] (mc, adjbox),
  Manuel Koller [aut] (mc, lmrob, psi-func.),
  Eduardo L. T. Conceicao [aut] (MM-, tau-, CM-, and MTL- nlrob),
  Maria Anna di Palma [ctb] (initial version of Comedian)
Maintainer: Martin Maechler <maechler@stat.math.ethz.ch>
Repository: CRAN
Date/Publication: 2024-09-27 15:30:02 UTC
Built: R 4.5.2; armv7--netbsdelf-eabihf; 2026-04-17 11:54:26 UTC; unix
